'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.context.context import Context
from vitu.account.account_manager import AccountManager
from vitu.trade.position.portfolio_position import PortfolioPosition

class Portfolio(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            self = super(Portfolio, cls).__new__(cls)
            cls._instance = self
        return cls._instance

    def __init__(self,strategy=None, start_date=None, end_date=None, commission=None,
                 frequency=None, refresh_rate=None, trigger_time=None):
        self.context = Context(self, start_date, end_date, commission, frequency, refresh_rate, trigger_time)
        self._accounts = AccountManager().create_accounts(self.context)
        self.strategy = strategy
        self._rebalance_history = dict()

        self.first_flag = True

    @property
    def accounts(self):
        return self._accounts

    @property
    def rebalance_history(self):
        return self._rebalance_history

    def get_account(self, name):
        """
        :param name: 'spot_account.okex'
        :return: account
        """
        return self.accounts[name]

    def record_history(self):
        pp_detail = dict()
        pp_total = dict()
        orders = list()
        all_total_profit = None
        last_profit_and_loss = None
        date = self.context.current_datetime()
        last_date = self.context.previous_datetime()

        for name, account in self.accounts.items():
            account = account.to_dict()
            orders.extend(account['orders'])
            account['orders'].clear()

        self.rebalance_history[date] = {
            'portfolio_position':{'detail':dict(),'total':dict()},
            'orders':dict()
        }

        for asset in self.context.asset_varieties:
            # TODO list to dict
            pp_detail[asset] = PortfolioPosition(self.context, asset, self.accounts).detail()
            # portfolio_position.append(pp_detail)
        all_total_profit = round(sum([i['total_amount'] for i in pp_detail.values()]), 4)

        # 各个持仓百分比
        for asset in self.context.asset_varieties:
            pp_detail[asset]['percentage_of_positions'] = round(pp_detail[asset]['total_amount']/all_total_profit, 4)


        if self.first_flag:
            last_profit_and_loss = 0
            for asset in self.context.asset_varieties:
                pp_detail[asset]["last_profit_and_loss"] = 0
            self.first_flag = False

        else:
            last_profit_and_loss = round((all_total_profit/self.rebalance_history[last_date]["portfolio_position"]['total']["profit"])-1, 4)
            for asset in self.context.asset_varieties:
                try:
                    profit_and_loss = round((pp_detail[asset]["total_amount"] /
                                             self.rebalance_history[last_date]["portfolio_position"]['detail'][asset]["total_amount"])-1,4)
                except:
                    profit_and_loss = 0
                pp_detail[asset]["last_profit_and_loss"] = profit_and_loss


        self.rebalance_history[date]['orders'] = orders


        pp_total = {"assets":self.context.asset_varieties,
                    "profit": all_total_profit,
                    "last_profit_and_loss": last_profit_and_loss}
        self.rebalance_history[date]["portfolio_position"]['detail'] = pp_detail
        self.rebalance_history[date]["portfolio_position"]['total'] = pp_total

        # print(self.rebalance_history)
        # logger.info('资产总价值：{}'.format(all_total_profit))









'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.data.static.min_order import min_order

def get_min_order(exchange,symbol):
    if exchange == 'poloniex':
        exchange = 'binance'
    min_order_qty = min_order[exchange][symbol.upper()]['min_order_qty']
    min_order_amount = min_order[exchange][symbol.upper()]['min_order_amount']
    return min_order_qty,min_order_amount

if __name__ == '__main__':
    # baseAsset            基础货币 BTC
    # quoteAsset           计价货币 USDT
    # maxMarketOrderQty    最大市价单数量
    # minMarketOrderQty    最小市价单数量
    # minTickSize          （最下价格波动）
    # minOrderValue        最小下单金额 10USDT
    # minTradeAmount       最小下单数量 0.000001BTC
    symbols = ['btc/usdt','eth/usdt','eos/usdt','xrp/usdt','eth/btc']
    for symbol in symbols:
        min_order_qty,min_order_amount = get_min_order('binance',symbol)
        print(min_order_qty,min_order_amount)











'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.trade.order.trade import Trade
from vitu.utils.trade_utils import trade_update_position

def match_engine(current_position, context, order):
    """
    :param account: account
    :param order: order
    :return: trade
    """
    # 撮合：每撮合一笔，创建一个trade,直到order数量撮合完/撤单为止
    # 产生Trade更新对应Order和Postion
    trade = Trade(context, order['id'], order['side'],order['limit_price'], order['initial_qty'])
    order.trade_update(trade)
    # trade_update_position(current_position, trade['side'], order['instrument'], trade['price'], trade['qty'])
    trade_update_position(current_position, order['instrument'], trade)













'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
class Logger(object):
    def __init__(self):
        self.current_date = None

    def debug(self, message):
        print(self.current_date, '[DEBUG]', message)

    def info(self, message):
        print(self.current_date, '[INFO]', message)

    def warning(self, message):
        print(self.current_date, '[WARNING]', message)

    def error(self, message):
        print(self.current_date, '[ERROR]', message)

    def critical(self, message):
        print(self.current_date, '[CRITICAL]', message)

logger = Logger()


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import datetime
import numpy as np

from vitu.trade.position.position import Position
from vitu.utils.date_utils import str2timestamp


class SpotPosition(Position):
    # __slots__ = [
    #     'available',
    #     'frozen',
    #     '_context'
    # ]
    def __init__(self, asset_class=None, asset=None, available=0, frozen=0, avg_cost_btc=0, avg_cost_usdt=0):
        """
        :param asset_class: 'spot'/'contract'
        :param asset: 'btc'/'eth' ...
        :param available: 可用数量
        :param frozen: 冻结数量
        :param avg_cost_btc: btc的持仓成本
        :param
        """
        super(SpotPosition, self).__init__(asset_class, asset)
        self.available = available
        self.frozen = frozen
        self.avg_cost_btc = avg_cost_btc
        self.avg_cost_usdt = avg_cost_usdt  # TDOD 哪来的None

        self._context = None

    @property
    def context(self):
        return self._context

    @context.setter
    def context(self, context):
        self._context = context

    @property
    def value(self):
        """
        :return: 持仓市值（随市场价实时变动）
        """
        if not self._context:
            return 0
        else:
            # date = self._context.clock.current_date
            # current_timestamp = str2timestamp(str(date.date()))  # 只关注date() time()舍掉
            # current_timestamp = str(datetime.datetime.fromtimestamp(current_timestamp))
            # cmc_key = 'cmc-spot-' + self.asset + 'usd'
            # df = self.context.cacher.data[cmc_key]
            # return df.loc[(df['timestamp'] == current_timestamp)]['close'].tolist()[0]

            try :
                freq1=self._context.frequency
                if freq1 in ['d','1d','day','1day']:
                    last_bar_index = self._context.clock.bars_start + self._context.clock.pre_bar  # date取日级数据
                elif freq1 in ['m','1m','min','1minute']:
                    last_bar_index = self._context.clock.bars_start + self._context.clock.pre_bar*1440
                elif freq1 in ['5m','5min','5minutes']:
                    last_bar_index = self._context.clock.bars_start + self._context.clock.pre_bar*288
                if self.asset == 'usdt':
                   return 1.00
                else:
                    if hasattr(self._context,'symbol'):
                        cmc_key = self._context.symbol.split('.')[1] + '-spot-' + self._context.symbol.split('.')[0].replace('/', '').lower()
                    else:
                        cmc_key=list(self._context.min_order.keys())[0]+ '-spot-' + self.asset+'usdt'
                    df = self.context.cacher.data[cmc_key]
                    return df.loc[last_bar_index,'close']
            except Exception as e:
                daybar=self._context.clock.day_bar+self._context.clock.pre_bar
                cmc_key = 'cmc-spot-' + self.asset + 'usd'
                df = self.context.cacher.data[cmc_key]
                return df.loc[daybar,'close']

    @property
    def total(self):
        """
        :return: 总数量
        """
        if not self.available + self.frozen:
            return 0
        else:
            return self.available + self.frozen
    @property
    def amount(self):
        """
        :return: 持仓浮动盈亏（随市场价实时变动）
        """
        if not self.value:
            return 0
        else:
            return self.total * self.value

    def order_update(self, currency_type, side, price, qty):
        """
        :param currency_type: base/quote  基础货币/计价货币
        :param side: buy/sell
        :param price: 价格
        :param qty: 数量
        """
        if side == 'buy':
            if currency_type == 'quote':
                self.available -= price * qty
                self.frozen += price * qty
        if side == 'sell':
            if currency_type == 'base':
                self.available -= qty
                self.frozen += qty

    def trade_update(self, currency_type, trade):
        """
        {
	    'id': '69327ec1-ea66-11e9-bea1-005056c00008',
	    'order_id': '69327ec0-ea66-11e9-8746-005056c00008',
	    'side': 'buy',
	    'price': 3432.88,
	    'qty': 145.65029,
	    'amount': 499999.967535,
	    'commission': 0.02913,
	    'create_time': '2018-12-10 00:00:00'
	    }
        :param currency_type: 'base'/'quote'  基础货币/计价货币
        :param trade: instance
        :param relative_currency: symol的另一个
        """
        # btc_usdt
        if trade['side'] == 'buy':
            if currency_type == 'base':
                self.available += (trade['qty'] - trade['commission'])
                # self.available += trade['qty']
            elif currency_type == 'quote':
                self.frozen -= trade['price']*trade['qty']

        if trade['side'] == 'sell':
            if currency_type == 'base':
                self.frozen -= trade['qty']
            elif currency_type == 'quote':
                self.available += (trade['price']*trade['qty']-trade['commission'])
                # self.available += trade['price']*trade['qty']

    def trade_update_cost(self, currency_type, trade, relative_currency):
        if not self.available:
            self.avg_cost_btc = 0
            self.avg_cost_usdt = 0
            return
        if trade['side'] == 'buy':
            if currency_type == 'base':
                self.avg_cost_btc = ((self.available-trade['qty']+trade['commission']) * self.avg_cost_btc +
                                     (relative_currency.avg_cost_btc * trade['amount'])) / self.available
                self.avg_cost_usdt = ((self.available-trade['qty']+trade['commission']) * self.avg_cost_usdt +
                                      (relative_currency.avg_cost_usdt * trade['amount'])) / self.available
            elif currency_type == 'quote':
                self.avg_cost_btc = self.avg_cost_btc
                self.avg_cost_usdt = self.avg_cost_usdt
        if trade['side'] == 'sell':
            if currency_type == 'base':
                self.avg_cost_btc = self.avg_cost_btc
                self.avg_cost_usdt = self.avg_cost_usdt
            elif currency_type == 'quote':
                self.avg_cost_btc = ((self.available-trade['amount']+trade['commission']) * self.avg_cost_btc +
                                     (relative_currency.avg_cost_btc * trade['qty'])) / self.available
                self.avg_cost_usdt = ((self.available-trade['amount']+trade['commission']) * self.avg_cost_usdt +
                                      (relative_currency.avg_cost_usdt * trade['qty'])) / self.available

    def detail(self):
        tempvalue=0
        tempamout=0
        tempvalue=round(self.value, 6)
        if tempvalue:
           tempamout=self.total * tempvalue

        return {
            'asset_class':self.asset_class,
            'asset':self.asset,
            'value':tempvalue, 
            'amount':round(tempamout, 6),
            'total':round(self.total, 6),
            'available':round(self.available, 6),
            'frozen':self.frozen,

            'avg_cost_btc':self.avg_cost_btc,
            'avg_cost_usdt':self.avg_cost_usdt
        }

    def to_dict(self):
        return {
            'asset': self.asset,
            'available': round(self.available, 6),
            'frozen': self.frozen,

            'avg_cost_btc': self.avg_cost_btc,
            'avg_cost_usdt': self.avg_cost_usdt
        }

    def __repr__(self):
        return "SpotPosition(asset_class: {}, asset: {}, amount: {}, value: {}, total: {}, available: {}, frozen: {}, " \
               "avg_cost_btc: {}, avg_cost_usdt: {})".format(
            self.asset_class, self.asset, round(self.amount, 4), round(self.value, 4), round(self.total, 4),
            round(self.available, 4), self.frozen, self.avg_cost_btc, self.avg_cost_usdt)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import os

class Config:

    @classmethod
    def redis_host(cls):
        return os.environ.get('REDIS_HOST') or 'localhost'

    @classmethod
    def redis_port(cls):
        return int(os.environ.get('REDIS_PORT') or '6379')

    @classmethod
    def h5_root_dir(cls):
        return os.environ.get("H5_ROOT_DIR") or '/opt/data/vitu/bundle'





'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.trade.position.position import Position

class ContractPosition(Position):
    __slots__ = [
        'available_balance',
        'side',
        'available_qty',
        'frozen_qty',
        'insurance',
        'leverage',
        'realised_pnl'
    ]
    def __init__(self, available_balance=None, asset_class=None, asset=None, side=None, available_qty=None,
                 frozen_qty=None,insurance=None, leverage=None, realised_pnl=None):
        """
        :param asset_class: 'spot'/'contract'
        :param asset: 'xbtT19' 合约标的
        :param available_balance: 账户余额
        :param side: long/short
        :param available_qty: 可用合约数量
        :param frozen_qty: 冻结合约数量
        :param insurance: 保证金
        :param leverage: 杠杆倍数
        :param realised_profit_and_lost: 已实现盈亏
        """
        super(ContractPosition, self).__init__(asset_class, asset)
        self.available_balance = available_balance
        self.side = side
        self.available_qty = available_qty
        self.frozen_qty = frozen_qty
        self.insurance = insurance
        self.leverage = leverage
        self.realised_pnl = realised_pnl

    @property
    def total_qty(self):
        return self.available_qty + self.frozen_qty

    def detail(self):
        return {
            'asset_class':self.asset_class,
            'asset':self.asset,
            'available_balance':self.available_balance,
            'side':self.side,
            'total_qty':self.total_qty,
            'available_qty':self.available_qty,
            'frozen_qty':self.frozen_qty,
            'insurance':self.insurance,
            'leverage': self.leverage,
            'realised_pnl': self.realised_pnl
        }

    def __repr__(self):
        return "FuturePosition(asset_class: {}, asset: {}, available_balance: {}, side: {}, available_qty: {}, " \
               "frozen_qty: {}, insurance: {}, leverage: {}, realised_pnl: {})".format(
            self.asset_class, self.asset, self.available_balance, self.side, self.available_qty, self.frozen_qty,
            self.insurance, self.leverage, self.realised_pnl)



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import math
import numpy as np

from vitu.trade.order.order import Order
from vitu.core.match_engine import match_engine
from vitu.utils.trade_utils import order_update_position
from vitu.utils.error_utils import Errors
from vitu.utils.log_utils import logger

class Account(object):
    __slots__ = [
        'context',
        'name',
        'exchange',
        'asset_class',
        'current_position',
        'history_orders',
        'orders'
    ]

    def __init__(self, context, name, exchange, asset_class, current_position):
        """
        :param name:              spot_account
        :param exchange:          binance/okex
        :param asset_class:       spot/contract
        :param current_position:  {'btc':Position,'eth':Position}
        """
        self.context = context
        self.name = name
        self.exchange = exchange
        self.asset_class = asset_class
        self.current_position = current_position     # key = 'btc'..., value = Position
        self.history_orders = dict()                    # key = id, value = Order
        self.orders = list()  # 存储今日订单

    def sell(self, symbol_exchange, price, qty):
        """
        :param symbol_exchange: "BTC/USDT.binance"
        :param price:  限价
        :param qty: 数量
        :return:
        """
        exchange = symbol_exchange.split('.')[1]
        symbol = symbol_exchange.split('.')[0]
        base_currency = symbol.split('/')[0].lower()   # btc  基础货币
        quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
        if not isinstance(price,(int,float)) or np.isnan(price):
            raise Errors.INVALID_PRICE
        if not isinstance(qty, (int, float)) or np.isnan(qty):
            raise Errors.INVALID_AMOUNT
        # 低于current_price才能卖出，高于current_price才能买入
        current_price = self.context.get_price(symbol_exchange)
        if price > current_price:
            logger.debug('限价单价格需要小于等于市场价才能卖出,卖出失败.')
            return
        if base_currency not in self.current_position.keys():
            # logger.debug('没有 {} 资产,卖出失败.'.format(base_currency))
            return

        qty = math.floor(qty*100000000)/100000000
        price = math.floor(price*100000000)/100000000
        amount = math.floor((qty*price)*100000000)/100000000

        # 判断是否有足够资金下单
        if self.current_position[base_currency].detail()['available'] < qty:
            # logger.debug('{} 不足,卖出失败.'.format(base_currency))
            return
        # 判断是否小于最下下单精度、最小下单金额
        min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
        min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
        if qty < min_order_qty:
            logger.debug('不足下单最小精度 {} {},卖出失败.'.format('%.6f'%min_order_qty,base_currency.upper()))
            return
        if amount < min_order_amount:
            logger.debug('不足下单最小金额 {} {},卖出失败.'.format('%.6f'%min_order_amount,quote_currency.upper()))
            return

        # 下单
        order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'sell', 'limit', price, qty)
        order_id = order['id']
        self.history_orders[order_id] = order
        order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
        match_engine(self.current_position, self.context, order)
        if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
            self.orders.append(order.detail())
        # logger.debug('{} 卖出成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
        # return order.detail()
        return order_id

    def buy(self, symbol_exchange, price, qty):
        """
        :param symbol_exchange: "BTC/USDT.binance"
        :param price:  限价
        :param qty: 数量
        :return:
        """
        exchange = symbol_exchange.split('.')[1]
        symbol = symbol_exchange.split('.')[0]
        base_currency = symbol.split('/')[0].lower()   # btc  基础货币
        quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
        if not isinstance(price,(int,float)) or np.isnan(price):
            raise Errors.INVALID_PRICE
        if not isinstance(qty, (int, float)) or np.isnan(qty):
            raise Errors.INVALID_AMOUNT
        # 低于current_price才能卖出，高于current_price才能买入
        current_price = self.context.get_price(symbol_exchange)
        if price < current_price:
            logger.debug('限价单价格需要大于等于市场价才能买入,买入失败.')
            return
        if quote_currency not in self.current_position.keys():
            logger.debug('没有 {} 资产,买入失败.'.format(quote_currency))
            return

        qty = math.floor(qty*100000000)/100000000
        price = math.floor(price*100000000)/100000000
        amount = math.floor((qty*price)*100000000)/100000000

        # 判断是否有足够资金下单
        if self.current_position[quote_currency].detail()['available'] < amount:
            # logger.debug('{} 不足,买入失败.'.format(quote_currency))
            return
        # 判断是否小于最下下单精度、最小下单金额
        min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
        min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
        if qty < min_order_qty:
            logger.debug('不足下单最小精度 {} {},买入失败.'.format(min_order_qty,base_currency.upper()))
            return
        if amount < min_order_amount:
            logger.debug('不足下单最小金额 {} {},买入失败.'.format(min_order_amount,quote_currency.upper()))
            return

        # 下单
        order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'buy', 'limit', price, qty)
        order_id = order['id']
        self.history_orders[order_id] = order
        order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
        match_engine(self.current_position, self.context, order)
        if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
            self.orders.append(order.detail())
        # logger.debug('{} 买入成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
        # return order.detail()
        return order_id

    # def sell_value(self, symbol_exchange, price, value):
    #     """
    #     :param symbol_exchange: "BTC/USDT.binance"
    #     :param price: 限价
    #     :param value: 卖出多少资产价值
    #     :return:
    #     """
    #     exchange = symbol_exchange.split('.')[1]
    #     symbol = symbol_exchange.split('.')[0]
    #     base_currency = symbol.split('/')[0].lower()  # btc  基础货币
    #     quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    #     if not isinstance(price, (int, float)) or np.isnan(price):
    #         raise Errors.INVALID_PRICE
    #     if not isinstance(value, (int, float)) or np.isnan(value):
    #         raise Errors.INVALID_AMOUNT
    #     # 低于current_price才能卖出，高于current_price才能买入
    #     current_price = self.context.get_price(symbol_exchange)
    #     if price > current_price:
    #         logger.warning('限价单价格需要小于等于市场价才能卖出,卖出失败.')
    #         return
    #     if base_currency not in self.current_position.keys():
    #         logger.warning('没有 {} 资产,卖出失败.'.format(base_currency))
    #         return
    #
    #     qty = math.floor((value/current_price) * 100000000) / 100000000
    #     price = math.floor(price*100000000)/100000000
    #     amount = math.floor((qty*price)*100000000)/100000000
    #
    #     # 判断是否有足够资金下单
    #     if self.current_position[base_currency].detail()['available'] < qty:
    #         logger.warning('{} 不足,卖出失败.'.format(base_currency))
    #         return
    #     # 判断是否小于最下下单精度、最小下单金额
    #     min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     if qty < min_order_qty:
    #         logger.warning('不足下单最小精度 {} {},卖出失败.'.format('%.6f'%min_order_qty,base_currency.upper()))
    #         return
    #     if amount < min_order_amount:
    #         logger.warning('不足下单最小金额 {} {},卖出失败.'.format('%.6f'%min_order_amount,quote_currency.upper()))
    #         return
    #
    #     # 下单
    #     order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'sell', 'limit', price, qty)
    #     order_id = order['id']
    #     self.history_orders[order_id] = order
    #     order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
    #     match_engine(self.current_position, self.context, order)
    #     if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
    #         self.orders.append(order.detail())
    #     logger.info('{} 卖出成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
    #     return order_id
    #
    # def buy_value(self, symbol_exchange, price, value):
    #     """
    #     :param symbol_exchange: "BTC/USDT.binance"
    #     :param price:  限价
    #     :param value: 价值
    #     :return:
    #     """
    #     exchange = symbol_exchange.split('.')[1]
    #     symbol = symbol_exchange.split('.')[0]
    #     base_currency = symbol.split('/')[0].lower()   # btc  基础货币
    #     quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    #     if not isinstance(price,(int,float)) or np.isnan(price):
    #         raise Errors.INVALID_PRICE
    #     if not isinstance(value, (int, float)) or np.isnan(value):
    #         raise Errors.INVALID_AMOUNT
    #     # 低于current_price才能卖出，高于current_price才能买入
    #     current_price = self.context.get_price(symbol_exchange)
    #     if price < current_price:
    #         logger.warning('限价单价格需要大于等于市场价才能买入,买入失败.')
    #         return
    #     if quote_currency not in self.current_position.keys():
    #         logger.warning('没有 {} 资产,买入失败.'.format(quote_currency))
    #         return
    #
    #     qty = math.floor((value/current_price) * 100000000) / 100000000
    #     price = math.floor(price*100000000)/100000000
    #     amount = math.floor((qty*price)*100000000)/100000000
    #
    #     # 判断是否有足够资金下单
    #     if self.current_position[quote_currency].detail()['available'] < amount:
    #         logger.warning('{} 不足,买入失败.'.format(quote_currency))
    #         return
    #     # 判断是否小于最下下单精度、最小下单金额
    #     min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     if qty < min_order_qty:
    #         logger.warning('不足下单最小精度 {} {},买入失败.'.format(min_order_qty,base_currency.upper()))
    #         return
    #     if amount < min_order_amount:
    #         logger.warning('不足下单最小金额 {} {},买入失败.'.format(min_order_amount,quote_currency.upper()))
    #         return
    #
    #     # 下单
    #     order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'buy', 'limit', price, qty)
    #     order_id = order['id']
    #     self.history_orders[order_id] = order
    #     order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
    #     match_engine(self.current_position, self.context, order)
    #     if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
    #         self.orders.append(order.detail())
    #     logger.info('{} 买入成功! 价格:{} 数量:{}'.format(base_currency,price,qty))
    #     return order_id
    #
    # def sell_target_value(self, symbol_exchange, price, target_value):
    #     """
    #     :param symbol_exchange: "BTC/USDT.binance"
    #     :param price: 限价
    #     :param target_value: 卖出到剩余多少资产价值
    #     :return:
    #     """
    #     exchange = symbol_exchange.split('.')[1]
    #     symbol = symbol_exchange.split('.')[0]
    #     base_currency = symbol.split('/')[0].lower()  # btc  基础货币
    #     quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    #     if not isinstance(price, (int, float)) or np.isnan(price):
    #         raise Errors.INVALID_PRICE
    #     if not isinstance(target_value, (int, float)) or np.isnan(target_value):
    #         raise Errors.INVALID_AMOUNT
    #     # 低于current_price才能卖出，高于current_price才能买入
    #     current_price = self.context.get_price(symbol_exchange)
    #     if price > current_price:
    #         logger.warning('限价单价格需要小于等于市场价才能卖出,卖出失败.')
    #         return
    #     if base_currency not in self.current_position.keys():
    #         logger.warning('没有 {} 资产,卖出失败.'.format(base_currency))
    #         return
    #
    #     available = self.current_position[base_currency].detail()['available']
    #
    #     value = abs(target_value-available)
    #     qty = math.floor((value / current_price) * 100000000) / 100000000
    #     print(available,target_value,qty)
    #     if qty < 0:
    #         logger.warning('{} 不足,卖出失败.'.format(base_currency))
    #         return
    #     price = math.floor(price*100000000)/100000000
    #     amount = math.floor((qty*price)*100000000)/100000000
    #
    #     # 判断是否有足够资金下单
    #     if self.current_position[base_currency].detail()['available'] < qty:
    #         logger.warning('{} 不足,卖出失败.'.format(base_currency))
    #         return
    #     # 判断是否小于最下下单精度、最小下单金额
    #     min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     if qty < min_order_qty:
    #         logger.warning('不足下单最小精度 {} {},卖出失败.'.format('%.6f'%min_order_qty,base_currency.upper()))
    #         return
    #     if amount < min_order_amount:
    #         logger.warning('不足下单最小金额 {} {},卖出失败.'.format('%.6f'%min_order_amount,quote_currency.upper()))
    #         return
    #
    #     # 下单
    #     order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'sell', 'limit', price, qty)
    #     order_id = order['id']
    #     self.history_orders[order_id] = order
    #     order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
    #     match_engine(self.current_position, self.context, order)
    #     if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
    #         self.orders.append(order.detail())
    #     logger.info('{} 卖出成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
    #     return order_id
    #
    # def buy_target_value(self, symbol_exchange, price, target_value):
    #     """
    #     :param symbol_exchange: "BTC/USDT.binance"
    #     :param price:  限价
    #     :param target_value: 买入到剩余多少价值
    #     :return:
    #     """
    #     exchange = symbol_exchange.split('.')[1]
    #     symbol = symbol_exchange.split('.')[0]
    #     base_currency = symbol.split('/')[0].lower()   # btc  基础货币
    #     quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    #     if not isinstance(price,(int,float)) or np.isnan(price):
    #         raise Errors.INVALID_PRICE
    #     if not isinstance(target_value, (int, float)) or np.isnan(target_value):
    #         raise Errors.INVALID_AMOUNT
    #     # 低于current_price才能卖出，高于current_price才能买入
    #     current_price = self.context.get_price(symbol_exchange)
    #     if price < current_price:
    #         logger.warning('限价单价格需要大于等于市场价才能买入,买入失败.')
    #         return
    #     if quote_currency not in self.current_position.keys():
    #         logger.warning('没有 {} 资产,买入失败.'.format(quote_currency))
    #         return
    #
    #     available = self.current_position[quote_currency].detail()['available']
    #     value = math.floor((available - target_value) * 100000000) / 100000000
    #     qty = math.floor((value / current_price) * 100000000) / 100000000
    #     print(available,target_value,qty)
    #
    #     if qty < 0:
    #         logger.warning('{} 不足,买入失败.'.format(quote_currency))
    #         return
    #     price = math.floor(price*100000000)/100000000
    #     amount = math.floor((qty*price)*100000000)/100000000
    #
    #     # 判断是否有足够资金下单
    #     if self.current_position[quote_currency].detail()['available'] < amount:
    #         logger.warning('{} 不足,买入失败.'.format(quote_currency))
    #         return
    #     # 判断是否小于最下下单精度、最小下单金额
    #     min_order_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     min_order_amount = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
    #     if qty < min_order_qty:
    #         logger.warning('不足下单最小精度 {} {},买入失败.'.format(min_order_qty,base_currency.upper()))
    #         return
    #     if amount < min_order_amount:
    #         logger.warning('不足下单最小金额 {} {},买入失败.'.format(min_order_amount,quote_currency.upper()))
    #         return
    #
    #     # 下单
    #     order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'buy', 'limit', price, qty)
    #     order_id = order['id']
    #     self.history_orders[order_id] = order
    #     order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
    #     match_engine(self.current_position, self.context, order)
    #     if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
    #         self.orders.append(order.detail())
    #     logger.info('{} 买入成功! 价格:{} 数量:{}'.format(base_currency,price,qty))
    #     return order_id
    #
    # def sell_target_pct(self, symbol_exchange, price, target_pct):
    #     """
    #     :param symbol_exchange: "BTC/USDT.binance"
    #     :param price: 限价
    #     :param target_pct: 卖出到资产百分比的数量
    #     :return:
    #     """
    #     exchange = symbol_exchange.split('.')[1]
    #     symbol = symbol_exchange.split('.')[0]
    #     base_currency = symbol.split('/')[0].lower()  # btc  基础货币
    #     quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
    #     if not isinstance(price, (int, float)) or np.isnan(price):
    #         raise Errors.INVALID_PRICE
    #     if not isinstance(target_pct, (int, float)) or np.isnan(target_pct):
    #         raise Errors.INVALID_AMOUNT
    #     # 低于current_price才能卖出，高于current_price才能买入
    #     current_price = self.context.get_price(symbol_exchange)
    #     if price > current_price:
    #         logger.warning('限价单价格需要小于等于市场价才能卖出,卖出失败.')
    #         return
    #     if base_currency not in self.current_position.keys():
    #         logger.warning('没有 {} 资产,卖出失败.'.format(base_currency))
    #         return
    #
    #     qty = math.floor((self.current_position[base_currency].detail()['available'] * (1-target_pct)) * 100000000) / 100000000
    #     price = math.floor(price*100000000)/100000000
    #     amount = math.floor((qty*price)*100000000)/100000000
    #
    #     # 下单
    #     order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'sell', 'limit', price, qty)
    #     order_id = order['id']
    #     self.history_orders[order_id] = order
    #     order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
    #     match_engine(self.current_position, self.context, order)
    #     if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
    #         self.orders.append(order.detail())
    #     logger.info('{} 卖出成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
    #     return order_id

    def sell_pct(self, symbol_exchange, price, pct):
        """
        :param symbol_exchange: "BTC/USDT.binance"
        :param price: 限价
        :param pct: 卖出资产百分比的数量
        :return:
        """
        exchange = symbol_exchange.split('.')[1]
        symbol = symbol_exchange.split('.')[0]
        base_currency = symbol.split('/')[0].lower()  # btc  基础货币
        # quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
        if not isinstance(price, (int, float)) or np.isnan(price):
            raise Errors.INVALID_PRICE
        if not isinstance(pct, (int, float)) or np.isnan(pct):
            raise Errors.INVALID_AMOUNT

        if base_currency not in self.current_position.keys():
            logger.debug('没有 {} 资产,卖出失败.'.format(base_currency))
            return
        base_min_qty = self.context.min_order[exchange][symbol.lower()]['min_order_qty']
        if self.current_position[base_currency].detail()['available'] < base_min_qty:
            logger.debug('{} 不足下单最小精度 {} {},卖出失败.'.format(base_currency,'%.6f'%base_min_qty,base_currency.upper()))
            return
        # 低于current_price才能卖出，高于current_price才能买入
        # current_price = self.context.get_price(symbol_exchange)
        # if price > current_price:
        #     logger.debug('限价单价格需要小于等于市场价才能卖出,卖出失败.')
        #     return

        qty = math.floor((self.current_position[base_currency].detail()['available']*pct)*1000000)/1000000
        price = math.floor(price*1000000)/1000000
        order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'sell', 'limit', price, qty)
        order_id = order['id']
        self.history_orders[order_id] = order
        order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
        match_engine(self.current_position, self.context, order)
        if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
            self.orders.append(order.detail())
        # logger.debug('{} 卖出成功! 价格:{} 数量:{}'.format(base_currency, price, qty))
        # return order.detail()
        return order_id

    def buy_pct(self, symbol_exchange, price, pct):
        """
        :param symbol_exchange: "BTC/USDT.binance"
        :param price:  限价
        :param pct: 买入资产百分比的数量
        :return:
        """
        exchange = symbol_exchange.split('.')[1]
        symbol = symbol_exchange.split('.')[0]
        base_currency = symbol.split('/')[0].lower()   # btc  基础货币
        quote_currency = symbol.split('/')[1].lower()  # usdt 计价货币
        if not isinstance(price,(int,float)) or np.isnan(price):
            raise Errors.INVALID_PRICE

        if quote_currency not in self.current_position.keys():
            logger.debug('没有 {} 资产,买入失败.'.format(quote_currency))
            return

        quote_min_amount = self.context.min_order[exchange][symbol.lower()]['min_order_amount']
        if self.current_position[quote_currency].detail()['available'] < quote_min_amount:
            logger.debug('{} 不足下单最小精度 {} {},买入失败.'.format(quote_currency,quote_min_amount,quote_currency.upper()))
            return
        qty = math.floor((self.current_position[quote_currency].detail()['available'] * pct)/price*1000000)/1000000
        if not isinstance(qty,(int,float)) or np.isnan(qty):
            raise Errors.INVALID_AMOUNT
        # 低于current_price才能卖出，高于current_price才能买入
        current_price = self.context.get_price(symbol_exchange)
        if price < current_price:
            logger.debug('限价单价格需要大于等于市场价才能买入,买入失败.')
            return

        price = math.floor(price*1000000)/1000000
        order = Order(self.context, self.name, symbol, self.exchange, self.asset_class, 'buy', 'limit', price, qty)
        order_id = order['id']
        self.history_orders[order_id] = order
        order_update_position(self.context, self.current_position, order['side'], exchange, symbol, price, qty)
        match_engine(self.current_position, self.context, order)
        if not self.orders or self.orders[0]['create_time'] == self.context.current_datetime():
            self.orders.append(order.detail())
        # logger.debug('{} 买入成功! 价格:{} 数量:{}'.format(base_currency,price,qty))
        # return order.detail()
        return order_id

    def get_positions(self):
        """
        :return: 获取仓位
        """
        return {asset:position.to_dict() for asset,position in self.current_position.items()}

    def get_position(self, asset):
        """
        :param asset: 币种
        :return: 获取指定asset仓位
        """
        asset = asset.lower()
        if asset not in self.current_position.keys():
            asset_position = {'asset':asset,'available':0,'frozen':0,'avg_cost_usdt':0,'avg_cost_btc':0}
            return asset_position
        else:
            return self.current_position[asset].to_dict()

    # def is_holding(self, asset):
    #     if asset.lower() in self.get_positions().keys():
    #         return True
    #     else:
    #         return False

    def get_order(self, order_id):
        """
        :param order_id: order订单号
        :return: 通过order_id找对应order详情
        """
        return self.history_orders[order_id]

    def get_orders(self, status):
        """
        :param status: 'Filled' | 'Partial_Filled' | 'Pending' | 'Cancelled' | 'Withdrawn'
        :return: 返回指定状态的所有order
        """
        return [order for order in self.history_orders.values() if order['status'] == status]

    def get_trades(self, order_id):
        """
        :param order_id: order订单号
        :return: 通过order_id找对应trades详情
        """
        return self.history_orders[order_id].trades

    @property
    def __dict__(self):
        return {key: self.__getattribute__(key) for key in self.__slots__}

    def to_dict(self):
        return self.__dict__

    def __repr__(self):
        return "Account(name: {}, exchange: {}, asset_class: {}, current_position: {}, open_orders: {})".format(
            self.name, self.exchange, self.asset_class, self.current_position, self.history_orders)

if __name__ == '__main__':
    pass


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
 '''
 
"""A setuptools based setup module for h5json.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
from setuptools import setup
from os import path
import os

here = path.abspath(path.dirname(__file__))
root_dir = 'vitu'
py_modules = set()
py_packages = set()

for root, dirs, files in os.walk(root_dir):
    for f in files:
        if f.endswith('.py'):
            if root == root_dir:
                py_modules.add(root + '.' + f.split('.')[0])

            if root != root_dir:
                py_packages.add(root.replace(os.path.sep, '.'))
py_packages.add('vitudata')
py_packages.add('vitudata.apis')            
print(py_modules)
print(py_packages)
setup(
    name='vitu',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='2.1.0',

    description='VITU/Halo - backtest framework',
    long_description='VITU/Halo - backtest framework',

    # The project's main homepage.
    url='http://vitu.ai',

    # Author details
    author='VituTech',
    author_email='service@vitutech.com',

    # Choose your license
    # license='BSD',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        # 'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    python_requires='>=3.0, <4',
    # What does your project relate to?
    keywords='vitu halo backtest',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    #packages=find_packages(exclude=['docs', 'test*']),
    packages=list(py_packages),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    py_modules=list(py_modules),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    setup_requires=['pkgconfig', 'six'],
    zip_safe=False,

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'vitu': ['wqy-microhei.ttc']
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[('vitu', ['vitu/wqy-microhei.ttc'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    
)



'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import datetime

def get_now_time():
    now_time = str(datetime.datetime.now()).split('.')[0]
    return now_time

def str2datetime(str_date):
    if len(str_date) == 10:
        return datetime.datetime.strptime(str_date, '%Y-%m-%d')
    elif len(str_date) == 19:
        return datetime.datetime.strptime(str_date, '%Y-%m-%d %H:%M:%S')
    elif len(str_date) == 26:
        return datetime.datetime.strptime(str_date, '%Y-%m-%dT%H:%M:%S.%f')
    elif len(str_date) == 28:
        str_date = str_date.split('+')[0]
        return datetime.datetime.strptime(str_date, '%Y-%m-%dT%H:%M:%S.%f')

def datetime2timestamp(date):
    return datetime.datetime.timestamp(date)

def str2timestamp(str_date):
    if len(str_date) == 10:
        return datetime.datetime.timestamp(datetime.datetime.strptime(str_date, '%Y-%m-%d'))
    elif len(str_date) == 19:
        return datetime.datetime.timestamp(datetime.datetime.strptime(str_date, '%Y-%m-%d %H:%M:%S'))
    elif len(str_date) == 26:
        return datetime.datetime.timestamp(datetime.datetime.strptime(str_date, '%Y-%m-%dT%H:%M:%S.%f'))
    elif len(str_date) == 28:
        str_date = str_date.split('+')[0]
        return datetime.datetime.timestamp(datetime.datetime.strptime(str_date, '%Y-%m-%dT%H:%M:%S.%f'))

def timestamp2str(timestamp):
    return datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')

def get_total_dates(frequency, refresh_rate, trigger_time, start, end):
    if trigger_time:
        start = start + ' ' + trigger_time[0]  # 只支持一个trigger_time
        end = end + ' ' + trigger_time[0]
    start_date = str2datetime(start)
    end_date = str2datetime(end)
    date = list()
    if frequency in ['d','1d','day','daily']:
        while True:
            if start_date <= end_date:
                date.append(str(start_date))
                start_date += datetime.timedelta(days=refresh_rate)
                # start_date += datetime.timedelta(days=1)
            else:
                break
    elif frequency in ['m','1m','min','minute']:
        while True:
            if start_date <= end_date:
                date.append(str(start_date))
                start_date += datetime.timedelta(minutes=refresh_rate)
                # start_date += datetime.timedelta(minutes=1)
            else:
                break
    elif frequency in ['5m','5min','5minutes']:
        while True:
            if start_date <= end_date:
                date.append(str(start_date))
                start_date += datetime.timedelta(minutes=5*refresh_rate)
                # start_date += datetime.timedelta(minutes=1)
            else:
                break
    return date
def get_day_dates(start_date, end_date,refresh_rate=1):
    start_date = str2datetime(start_date)
    end_date = str2datetime(end_date)
    daydate_list = list()
    while True:
        if start_date <= end_date:
            daydate_list.append(str(start_date))
            start_date += datetime.timedelta(days=refresh_rate)
        else:
            break
    return daydate_list

def get_total_timestamps(start_date, end_date):
    start_date = str2datetime(start_date)
    end_date = str2datetime(end_date)
    timestamp_list = list()
    while True:
        if start_date <= end_date:
            timestamp_list.append(round(datetime.datetime.timestamp(start_date)))
            start_date += datetime.timedelta(days=1)
        else:
            break
    return timestamp_list

def get_dates_length(start, end):
    start_date = str2datetime(start)
    end_date = str2datetime(end)
    length = end_date - start_date
    return length.days+1


if __name__ == '__main__':
    start = '2019-02-05'
    end = '2019-02-08'
    a = str2datetime(start)
    print(a.date())




import os
from matplotlib.font_manager import FontProperties
# from vitu.data_api import API
from vitu.configuration import Config
from vitu.utils.log_utils import logger

# api = API(Config.tushare_token())s
log = logger

font = None
dirName = os.path.dirname(os.path.abspath(__file__))
font = FontProperties(fname=os.path.join(dirName, 'wqy-microhei.ttc'), size=10)


'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
error = (lambda code, message: {'code': code, 'data': message})

class BacktestInputError(Exception):
    """回测参数输入异常"""
    pass

class BacktestError(Exception):
    """回测异常"""
    pass

class Errors(object):
    """
    Enumerate errors.
    """
    INVALID_START_DATE = BacktestInputError(error(500, '回测 start 日期不合法'))
    INVALID_END_DATE = BacktestInputError(error(500, '回测 end 不合法'))
    INVALID_START_END = BacktestInputError(error(500, '开始日期晚于结束日期'))
    INVALID_DATE = BacktestInputError(error(500, '日期不合法'))
    INVALID_TRADING_DAYS = BacktestInputError(error(500, '回测区间日期索引不到'))
    INVALID_FREQ = BacktestInputError(error(500, '回测 freq 不合法'))

    INVALID_PRICE = BacktestError(error(500, '下单价格不合法'))
    INVALID_AMOUNT = BacktestError(error(500, '下单数量不合法'))





'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
import time
import datetime

from vitu.utils.date_utils import (
    str2datetime,
    str2timestamp,
    datetime2timestamp
)

class Clock(object):
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            _instance = super(Clock, cls).__new__(cls)
            cls._instance = _instance
        return cls._instance

    def __init__(self, start_date=None, end_date=None, frequency=None, refresh_rate=None, trigger_time=None):
        """
        :param start_date: 初始日期
        :param end_date: 结束日期
        """
        self._str_date = start_date + ' ' + trigger_time[0] if trigger_time else start_date
        self._current_date = str2datetime(self._str_date)   # 当前日期 datetime
        self._previous_date = None
        self._current_timestamp = None                       # 当前日期 时间戳
        self._end_timestamp = str2timestamp(end_date)     # 结束日期 时间戳
        self._bars = 0 
        self._bars_m = 0                                     # 开始bars数量
        self._bars_start=0
        self._day_bar=0
        self._run_start = time.time()                        # 记录run运行时间
        self._datestart = time.time()
        self._run_end = None

        self._frequency = frequency
        self._refresh_rate = refresh_rate
  
    @property
    def bars(self):
        return self._bars

    @property
    def bars_m(self):
        return self._bars_m
    
    @property
    def bars_start(self):
        return self._bars_start

    @property
    def day_bar(self):
        return self._day_bar    
    
    @property
    def datestart(self):
        return self._datestart
    

    @bars.setter
    def bars(self, bars):
        self._bars = bars

    @property
    def current_date(self):
        return self._current_date

    @current_date.setter
    def current_date(self, current_date):
        self._current_date = current_date
    
    @property
    def pre_bar(self):
        if self._frequency in ['d','1d','day','daily']:
           self._prebar=300 
        elif self._frequency in ['m','1m','5m','min','minute','5min','5minutes']: 
           self._prebar=5  
        return self._prebar  
    @property
    def previous_date(self):
        if self._frequency in ['d','1d','day','daily']:
            self._previous_date = self._current_date - datetime.timedelta(days=self._refresh_rate)
            # self._previous_date = self._current_date - datetime.timedelta(days=1)
        elif self._frequency in ['m','1m','min','minute']:
            self._previous_date = self._current_date - datetime.timedelta(minutes=self._refresh_rate)
        elif self._frequency in ['5m','5min','5minutes']:
            self._previous_date = self._current_date - datetime.timedelta(minutes=5*self._refresh_rate)
        return self._previous_date

    @property
    def current_timestamp(self):
        self._current_timestamp = datetime2timestamp(self._current_date)
        return self._current_timestamp

    @property
    def end_timestamp(self):
        return self._end_timestamp

    @property
    def run_start(self):
        return self._run_start

    @property
    def run_end(self):
        return self._run_end

    @run_end.setter
    def run_end(self, timestamp):
        self._run_end = timestamp


    def reset_bars(self):
        self._bars = 0

    def reset_bars_m(self):
        self._bars_m = 0    

    def reset_datestart(self):
        self._datestart=time.time()

    def next(self):
        if self._frequency in ['d','1d','day','daily']:
            # self._current_date += datetime.timedelta(days=self._refresh_rate)
            self._current_date += datetime.timedelta(days=self._refresh_rate)
        elif self._frequency in ['m','1m','min','minute']:
            self._current_date += datetime.timedelta(minutes=self._refresh_rate)
        elif self._frequency in ['5m','5min','5minutes']:
            self._current_date += datetime.timedelta(minutes=5*self._refresh_rate)
        current_timestamp = str2timestamp(str(self._current_date.date()))  # 只关注date() time()舍掉
        current_timestamp = str(datetime.datetime.fromtimestamp(current_timestamp))
        if str(self._current_date) == current_timestamp :
            self._day_bar +=1
        self._bars += 1
        self._bars_m +=1
        self._bars_start+=1
        





'''/*---------------------------------------------------------------------------------------------
 *  Copyright (c) VituTech. All rights reserved.
 *  Licensed under the Apache License 2.0. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'''
from vitu.trade.position.spot_position import SpotPosition
from vitu.trade.position.contract_position import ContractPosition
from vitu.account.account import Account
from vitu.utils.data_utils import get_btc_usdt_cost


class AccountInfo(object):
    __slots__ = [
        'name',
        'exchange',
        'account_type',
        'position_base'
    ]
    def __init__(self, name, exchange, account_type, position_base):
        self.name = name
        self.exchange = exchange
        self.account_type = account_type
        self.position_base = position_base

    def detail(self):
        return {
            'name': self.name,
            'exchange': self.exchange,
            'account_type': self.account_type,
            'position_base': self.position_base
        }

    def __getitem__(self, key, default=None):
        item_value = self.__getattribute__(key) if self.__getattribute__(key) else default
        return item_value

    def __repr__(self):
        content = ', '.join(['{}: {{}}'.format(item) for item in self.__slots__]).format(
            self.name,self.exchange,self.account_type, self.position_base)
        return 'AccountInfo({})'.format(content)



class AccountManager(object):
    __instance = None
    __has_init = False
    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def __init__(self):  # 单例模式的初始化也应该只执行一次
        if AccountManager.__has_init == False:
            self.accounts_info = []
            self._accounts_name = None  # 记录account_name
            self.asset_varieties = None
            AccountManager.__has_init = True

    def config(self, name, exchange, account_type, position_base):
        """
        :param name : spot_account:
        :param exchange : binance
        :param account_type : digital.spot/contract
        :param position_base : ({'asset': 'BTC', 'qty': 100}, {'asset': 'USDT', 'qty': 200000}))
                               ({'asset': 'XBT/USD.long', 'qty': 100, 'leverage': 100, 'insurance': 2})
        """
        self.accounts_info.append(AccountInfo(name, exchange, account_type, position_base))

    def create_accounts(self, context):
        self._accounts_name = list()
        self.asset_varieties = list()

        accounts = dict()
        for account_info in self.accounts_info:
            name = account_info['name']
            self._accounts_name.append(name)  # 添加account_name
            exchange = account_info['exchange']
            asset_class = account_info['account_type'].split('.')[1]
            current_position = dict()
            if asset_class == 'spot':
                for i in account_info['position_base']:
                    asset = i['asset'].lower()
                    # 计算初始持仓成本
                    avg_cost_btc, avg_cost_usdt = get_btc_usdt_cost(asset, context.current_datetime())
                    if asset not in self.asset_varieties:
                        self.asset_varieties.append(asset)  # 添加asset
                    available = i['qty']
                    frozen = 0
                    current_position[asset] = SpotPosition(asset_class, asset, available, frozen, avg_cost_btc, avg_cost_usdt)
            if asset_class == 'contract':
                for i in account_info['position_base']:
                    asset = i['asset'].lower().replace('.','_')
                    if asset not in self.asset_varieties:
                        self.asset_varieties.append(asset)  # 添加asset
                    available = i['qty']
                    frozen = 0
                    leverage = i['leverage']
                    insurance = i['insurance']
                    current_position[asset] = ContractPosition(asset_class, asset, available, frozen, leverage, insurance)

            accounts[name] = Account(context, name, exchange, asset_class, current_position)

        context.accounts_name = self._accounts_name
        context.asset_varieties = self.asset_varieties
        return accounts

    def __repr__(self):
        content = ', '.join(['{}: {{}}'.format(item) for item in self.__slots__]).format(self.accounts_info)
        return 'AccountManager({})'.format(content)